from wagtail.blocks import StructValue

from wagtail_cblocks.factories import HeadingBlockFactory, ParagraphBlockFactory


def test_heading_block_factory():
    assert HeadingBlockFactory(text="Another title", level=4) == StructValue(
        None, [('text', "Another title"), ('level', 4)]
    )


def test_paragraph_block_factory():
    assert ParagraphBlockFactory(value="<p>A paragraph.</p>").source == (
        "<p>A paragraph.</p>"
    )



from django.core.exceptions import ImproperlyConfigured

import pytest
from bs4 import BeautifulSoup
from pytest_django.asserts import assertHTMLEqual
from wagtail_factories import DocumentFactory, ImageFactory, PageFactory

from wagtail_cblocks import blocks
from wagtail_cblocks.blocks import CSSClassMixin, Style, StylizedStructBlock

from .models import BaseBlock, ColumnsBlock, RowColumnsBlock


class BlockTest:
    def render(self, data, block=None):
        if block is None:
            block = self.block
        return block.render(block.to_python(data))

    def render_html(self, *args, **kwargs):
        return BeautifulSoup(self.render(*args, **kwargs), 'html.parser')


class TestCSSClassMixin:
    class Block(CSSClassMixin, blocks.ParagraphBlock):
        pass

    def test_in_meta(self):
        class Block(self.Block):
            class Meta:
                css_class = 'lead'

        block = Block()
        context = block.get_context(block.to_python('<p>Paragraph</p>'))
        assert context['css_class'] == 'lead'

    @pytest.mark.parametrize('value', ('lead', {'lead'}, ['lead']))
    def test_at_init(self, value):
        block = self.Block(css_class=value)
        context = block.get_context(block.to_python('<p>Paragraph</p>'))
        assert context['css_class'] == 'lead'

    def test_empty(self):
        block = self.Block()
        context = block.get_context(block.to_python('<p>Paragraph</p>'))
        assert context['css_class'] == ''


class TestStylizedStructBlock:
    STYLES = [
        Style('foo', "Foo", 'is-foo'),
        Style('bar', "Bar", 'is-bar'),
    ]

    def test_empty_styles(self):
        block = StylizedStructBlock()
        assert 'style' not in block.child_blocks
        context = block.get_context(block.to_python({}))
        assert context['css_class'] == ''

    def test_styles_from_property(self):
        class Block(StylizedStructBlock):
            styles = self.STYLES

        value = Block().to_python({'style': 'foo'})
        assert value.get('style').name == 'foo'

    def test_style(self):
        block = StylizedStructBlock(styles=self.STYLES)
        value = block.to_python({'style': 'foo'})
        assert block.get_prep_value(value) == {'style': 'foo'}
        assert isinstance(value['style'], Style)
        assert value['style'].name == 'foo'
        context = block.get_context(value)
        assert context['css_class'] == 'is-foo'

    def test_style_with_css_class(self):
        block = StylizedStructBlock(styles=self.STYLES, css_class='block')
        context = block.get_context(block.to_python({'style': 'foo'}))
        assert context['css_class'] == 'block is-foo'

    def test_default_style(self):
        block = StylizedStructBlock(styles=self.STYLES, default_style='bar')
        value = block.to_python({})
        assert isinstance(value.get('style'), Style)
        assert value.get('style').name == 'bar'
        context = block.get_context(value)
        assert context['css_class'] == 'is-bar'

    def test_default_style_object(self):
        block = StylizedStructBlock(
            styles=self.STYLES, default_style=self.STYLES[1]
        )
        value = block.to_python({})
        assert isinstance(value.get('style'), Style)
        assert value.get('style').name == 'bar'

    def test_unknown_style(self):
        block = StylizedStructBlock(styles=self.STYLES, default_style='bar')
        value = block.to_python({'style': 'baz'})
        assert value.get('style') is None
        context = block.get_context(value)
        assert context['css_class'] == ''


class TestHeadingBlock(BlockTest):
    block = blocks.HeadingBlock()

    def test_render(self):
        assertHTMLEqual(
            self.render({'text': "Un titre !", 'level': 2}),
            '<h2 id="un-titre">Un titre !</h2>',
        )


class TestParagraphBlock(BlockTest):
    block = blocks.ParagraphBlock()

    def test_features(self):
        assert self.block.features == blocks.ParagraphBlock.features

        block = blocks.ParagraphBlock(features=['bold', 'italic'])
        assert block.features == ['bold', 'italic']

    def test_render(self):
        data = '<p><i>Un</i> paragraphe !</p>'
        assertHTMLEqual(self.render(data), data)


@pytest.mark.django_db
class TestLinkBlock:
    block = blocks.LinkBlock()

    def get_value(self, target):
        return self.block.to_python({'target': target})

    def test_local_blocks(self):
        block = blocks.LinkBlock([('text', blocks.ParagraphBlock())])
        assert list(block.child_blocks.keys()) == ['text', 'target']

    def test_required(self):
        block = blocks.LinkBlock(required=False)
        assert block.required is False
        assert block.child_blocks['target'].required is False

        assert self.block.required is True
        assert self.block.child_blocks['target'].required is True

    def test_href_value_page(self, root_page):
        page = PageFactory(parent=root_page, title="About", slug='about')
        value = self.get_value([{'type': 'page', 'value': page.id}])
        assert value.href == '/about/'

    def test_href_value_page_invalid(self):
        value = self.get_value([{'type': 'page', 'value': 1000}])
        assert value.href is None

    def test_href_value_document(self):
        document = DocumentFactory()
        value = self.get_value([{'type': 'document', 'value': document.id}])
        assert value.href == document.file.url

    def test_href_value_document_invalid(self):
        value = self.get_value([{'type': 'document', 'value': 1000}])
        assert value.href is None

    def test_href_value_image(self):
        image = ImageFactory()
        value = self.get_value([{'type': 'image', 'value': image.id}])
        assert value.href == image.file.url

    def test_href_value_image_invalid(self):
        value = self.get_value([{'type': 'image', 'value': 1000}])
        assert value.href is None

    def test_href_value_external_url(self):
        url = 'http://example.org/truc/'
        value = self.get_value([{'type': 'url', 'value': url}])
        assert value.href == url

    def test_href_value_anchor(self):
        anchor = '#truc'
        value = self.get_value([{'type': 'anchor', 'value': anchor}])
        assert value.href == anchor

    def test_href_value_empty(self):
        value = self.get_value([])
        assert value.href is None

    def test_href_value_no_target(self):
        block = blocks.LinkBlock([('text', blocks.ParagraphBlock())])
        value = block.to_python({'text': "some text"})
        assert value.href is None


class TestButtonBlock(BlockTest):
    block = blocks.ButtonBlock()

    def test_render_link(self):
        url = 'http://example.org/truc/'
        assertHTMLEqual(
            self.render(
                {
                    'text': "Lien",
                    'link': {'target': [{'type': 'url', 'value': url}]},
                }
            ),
            '<a class="btn mb-3" href="{}">Lien</a>'.format(url),
        )


@pytest.mark.django_db
class TestImageBlock(BlockTest):
    block = blocks.ImageBlock()

    def test_render(self):
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'caption': '',
                'link': {'target': []},
            }
        )
        assert len(html.select('img.figure-img.img-fluid.mb-0')) == 1
        assert not html.select('figcaption')
        assert not html.select('a')

    def test_render_with_caption(self):
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'caption': "Une légende en dessous.",
                'link': {'target': []},
            }
        )
        assert html.select_one('figcaption').text == "Une légende en dessous."
        assert not html.select('a')

    def test_render_with_link(self):
        url = 'http://example.org/truc/'
        html = self.render_html(
            {
                'image': ImageFactory().id,
                'caption': '',
                'link': {'target': [{'type': 'url', 'value': url}]},
            }
        )
        assert html.select_one('a').attrs['href'] == url
        assert not html.select('figcaption')


class TestColumnsBlock(BlockTest):
    block = blocks.ColumnsBlock(BaseBlock())

    def test_columns_block_order(self):
        assert list(self.block.child_blocks.keys())[0] == 'columns'

    def test_render(self):
        url = 'http://example.org/truc/'
        data = {
            'columns': [
                [
                    {
                        'type': 'button_block',
                        'value': {
                            'text': "Lien",
                            'link': {
                                'target': [{'type': 'url', 'value': url}],
                            },
                            'style': 'primary',
                        },
                    },
                    {
                        'type': 'paragraph_block',
                        'value': "<p>A first paragraph.</p>",
                    },
                ],
                [
                    {
                        'type': 'paragraph_block',
                        'value': "<p>Another paragraph.</p>",
                    },
                ],
            ],
            'horizontal_align': 'center',
        }
        assertHTMLEqual(
            self.render(data),
            (
                '<div class="row text-center">'
                '<div class="col-sm">{}{}</div>'
                '<div class="col-sm">{}</div>'
                '</div>'
            ).format(
                '<a class="btn btn-primary mb-3" href="{}">Lien</a>'.format(
                    url
                ),
                '<p>A first paragraph.</p>',
                '<p>Another paragraph.</p>',
            ),
        )

    def test_sublcass_render(self):
        data = {
            'columns': [
                [
                    {
                        'type': 'paragraph_block',
                        'value': "<p>A first paragraph.</p>",
                    },
                ],
            ],
            'layout': 'auto',
        }
        assertHTMLEqual(
            self.render(data, RowColumnsBlock()),
            (
                '<div class="row row-cols-auto">'
                '<div class="col"><p>A first paragraph.</p></div>'
                '</div>'
            ),
        )

    def test_required_column_block(self):
        with pytest.raises(ImproperlyConfigured):

            class DummyColumnsBlock(ColumnsBlock):
                pass

            DummyColumnsBlock()



from django.templatetags.static import static
from django.utils.html import format_html

from wagtail import hooks


@hooks.register('insert_editor_css')
def editor_css():
    return format_html(
        '<link rel="stylesheet" href="{}">',
        static('wagtail_cblocks/css/editor.css'),
    )



import factory
import wagtail_factories

from . import blocks


class HeadingBlockFactory(wagtail_factories.StructBlockFactory):
    text = "A title"
    level = 3

    class Meta:
        model = blocks.HeadingBlock


class ParagraphBlockFactory(factory.Factory):
    # FIXME: In waiting of https://github.com/wagtail/wagtail-factories/pull/25

    value = "<p>Some <b>formatted</b> text.</p>"

    class Meta:
        model = blocks.ParagraphBlock

    @classmethod
    def _build(cls, model_class, value):
        return model_class().to_python(value)

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        return cls._build(model_class, *args, **kwargs)



# Generated by Django 4.1.7 on 2023-03-30 11:38

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.documents.blocks
import wagtail.fields
import wagtail.images.blocks
import wagtail_cblocks.blocks


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('wagtailcore', '0078_referenceindex'),
    ]

    operations = [
        migrations.CreateModel(
            name='StandardPage',
            fields=[
                (
                    'page_ptr',
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to='wagtailcore.page',
                    ),
                ),
                (
                    'body',
                    wagtail.fields.StreamField(
                        [
                            (
                                'title_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'text',
                                            wagtail.blocks.CharBlock(
                                                form_classname='title',
                                                label='Title',
                                            ),
                                        ),
                                        (
                                            'level',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    (2, 'Level 2 heading'),
                                                    (3, 'Level 3 heading'),
                                                    (4, 'Level 4 heading'),
                                                    (5, 'Level 5 heading'),
                                                ],
                                                label='Level',
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                'paragraph_block',
                                wagtail_cblocks.blocks.ParagraphBlock(),
                            ),
                            (
                                'button_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'text',
                                            wagtail.blocks.CharBlock(
                                                label='Text'
                                            ),
                                        ),
                                        (
                                            'link',
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        'target',
                                                        wagtail.blocks.StreamBlock(
                                                            [
                                                                (
                                                                    'page',
                                                                    wagtail.blocks.PageChooserBlock(
                                                                        icon='doc-empty-inverse',
                                                                        label='Page',
                                                                    ),
                                                                ),
                                                                (
                                                                    'document',
                                                                    wagtail.documents.blocks.DocumentChooserBlock(
                                                                        icon='doc-full',
                                                                        label='Document',
                                                                    ),
                                                                ),
                                                                (
                                                                    'image',
                                                                    wagtail.images.blocks.ImageChooserBlock(
                                                                        label='Image'
                                                                    ),
                                                                ),
                                                                (
                                                                    'url',
                                                                    wagtail.blocks.URLBlock(
                                                                        label='External link'
                                                                    ),
                                                                ),
                                                                (
                                                                    'anchor',
                                                                    wagtail.blocks.CharBlock(
                                                                        help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                        label='Anchor link',
                                                                    ),
                                                                ),
                                                            ],
                                                            required=True,
                                                        ),
                                                    )
                                                ],
                                                required=True,
                                            ),
                                        ),
                                        (
                                            'style',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    ('primary', 'Primary'),
                                                    ('secondary', 'Secondary'),
                                                    (
                                                        'primary-lg',
                                                        'Large primary',
                                                    ),
                                                ]
                                            ),
                                        ),
                                    ],
                                    default_style='primary-lg',
                                ),
                            ),
                            (
                                'image_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'image',
                                            wagtail.images.blocks.ImageChooserBlock(
                                                label='Image'
                                            ),
                                        ),
                                        (
                                            'caption',
                                            wagtail.blocks.CharBlock(
                                                label='Caption', required=False
                                            ),
                                        ),
                                        (
                                            'link',
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        'target',
                                                        wagtail.blocks.StreamBlock(
                                                            [
                                                                (
                                                                    'page',
                                                                    wagtail.blocks.PageChooserBlock(
                                                                        icon='doc-empty-inverse',
                                                                        label='Page',
                                                                    ),
                                                                ),
                                                                (
                                                                    'document',
                                                                    wagtail.documents.blocks.DocumentChooserBlock(
                                                                        icon='doc-full',
                                                                        label='Document',
                                                                    ),
                                                                ),
                                                                (
                                                                    'image',
                                                                    wagtail.images.blocks.ImageChooserBlock(
                                                                        label='Image'
                                                                    ),
                                                                ),
                                                                (
                                                                    'url',
                                                                    wagtail.blocks.URLBlock(
                                                                        label='External link'
                                                                    ),
                                                                ),
                                                                (
                                                                    'anchor',
                                                                    wagtail.blocks.CharBlock(
                                                                        help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                        label='Anchor link',
                                                                    ),
                                                                ),
                                                            ],
                                                            required=False,
                                                        ),
                                                    )
                                                ],
                                                required=False,
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                'hero_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'blocks',
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        'title_block',
                                                        wagtail.blocks.StructBlock(
                                                            [
                                                                (
                                                                    'text',
                                                                    wagtail.blocks.CharBlock(
                                                                        form_classname='title',
                                                                        label='Title',
                                                                    ),
                                                                ),
                                                                (
                                                                    'level',
                                                                    wagtail.blocks.ChoiceBlock(
                                                                        choices=[
                                                                            (
                                                                                2,
                                                                                'Level 2 heading',
                                                                            ),
                                                                            (
                                                                                3,
                                                                                'Level 3 heading',
                                                                            ),
                                                                            (
                                                                                4,
                                                                                'Level 4 heading',
                                                                            ),
                                                                            (
                                                                                5,
                                                                                'Level 5 heading',
                                                                            ),
                                                                        ],
                                                                        label='Level',
                                                                    ),
                                                                ),
                                                            ]
                                                        ),
                                                    ),
                                                    (
                                                        'paragraph_block',
                                                        wagtail_cblocks.blocks.ParagraphBlock(),
                                                    ),
                                                    (
                                                        'button_block',
                                                        wagtail.blocks.StructBlock(
                                                            [
                                                                (
                                                                    'text',
                                                                    wagtail.blocks.CharBlock(
                                                                        label='Text'
                                                                    ),
                                                                ),
                                                                (
                                                                    'link',
                                                                    wagtail.blocks.StructBlock(
                                                                        [
                                                                            (
                                                                                'target',
                                                                                wagtail.blocks.StreamBlock(
                                                                                    [
                                                                                        (
                                                                                            'page',
                                                                                            wagtail.blocks.PageChooserBlock(
                                                                                                icon='doc-empty-inverse',
                                                                                                label='Page',
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'document',
                                                                                            wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                icon='doc-full',
                                                                                                label='Document',
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'image',
                                                                                            wagtail.images.blocks.ImageChooserBlock(
                                                                                                label='Image'
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'url',
                                                                                            wagtail.blocks.URLBlock(
                                                                                                label='External link'
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'anchor',
                                                                                            wagtail.blocks.CharBlock(
                                                                                                help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                label='Anchor link',
                                                                                            ),
                                                                                        ),
                                                                                    ],
                                                                                    required=True,
                                                                                ),
                                                                            )
                                                                        ],
                                                                        required=True,
                                                                    ),
                                                                ),
                                                                (
                                                                    'style',
                                                                    wagtail.blocks.ChoiceBlock(
                                                                        choices=[
                                                                            (
                                                                                'primary',
                                                                                'Primary',
                                                                            ),
                                                                            (
                                                                                'secondary',
                                                                                'Secondary',
                                                                            ),
                                                                            (
                                                                                'primary-lg',
                                                                                'Large primary',
                                                                            ),
                                                                        ]
                                                                    ),
                                                                ),
                                                            ],
                                                            default_style='primary-lg',
                                                        ),
                                                    ),
                                                    (
                                                        'image_block',
                                                        wagtail.blocks.StructBlock(
                                                            [
                                                                (
                                                                    'image',
                                                                    wagtail.images.blocks.ImageChooserBlock(
                                                                        label='Image'
                                                                    ),
                                                                ),
                                                                (
                                                                    'caption',
                                                                    wagtail.blocks.CharBlock(
                                                                        label='Caption',
                                                                        required=False,
                                                                    ),
                                                                ),
                                                                (
                                                                    'link',
                                                                    wagtail.blocks.StructBlock(
                                                                        [
                                                                            (
                                                                                'target',
                                                                                wagtail.blocks.StreamBlock(
                                                                                    [
                                                                                        (
                                                                                            'page',
                                                                                            wagtail.blocks.PageChooserBlock(
                                                                                                icon='doc-empty-inverse',
                                                                                                label='Page',
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'document',
                                                                                            wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                icon='doc-full',
                                                                                                label='Document',
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'image',
                                                                                            wagtail.images.blocks.ImageChooserBlock(
                                                                                                label='Image'
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'url',
                                                                                            wagtail.blocks.URLBlock(
                                                                                                label='External link'
                                                                                            ),
                                                                                        ),
                                                                                        (
                                                                                            'anchor',
                                                                                            wagtail.blocks.CharBlock(
                                                                                                help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                label='Anchor link',
                                                                                            ),
                                                                                        ),
                                                                                    ],
                                                                                    required=False,
                                                                                ),
                                                                            )
                                                                        ],
                                                                        required=False,
                                                                    ),
                                                                ),
                                                            ]
                                                        ),
                                                    ),
                                                    (
                                                        'columns_block',
                                                        wagtail.blocks.StructBlock(
                                                            [
                                                                (
                                                                    'columns',
                                                                    wagtail.blocks.ListBlock(
                                                                        wagtail.blocks.StreamBlock(
                                                                            [
                                                                                (
                                                                                    'title_block',
                                                                                    wagtail.blocks.StructBlock(
                                                                                        [
                                                                                            (
                                                                                                'text',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    form_classname='title',
                                                                                                    label='Title',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'level',
                                                                                                wagtail.blocks.ChoiceBlock(
                                                                                                    choices=[
                                                                                                        (
                                                                                                            2,
                                                                                                            'Level 2 heading',
                                                                                                        ),
                                                                                                        (
                                                                                                            3,
                                                                                                            'Level 3 heading',
                                                                                                        ),
                                                                                                        (
                                                                                                            4,
                                                                                                            'Level 4 heading',
                                                                                                        ),
                                                                                                        (
                                                                                                            5,
                                                                                                            'Level 5 heading',
                                                                                                        ),
                                                                                                    ],
                                                                                                    label='Level',
                                                                                                ),
                                                                                            ),
                                                                                        ]
                                                                                    ),
                                                                                ),
                                                                                (
                                                                                    'paragraph_block',
                                                                                    wagtail_cblocks.blocks.ParagraphBlock(),
                                                                                ),
                                                                                (
                                                                                    'button_block',
                                                                                    wagtail.blocks.StructBlock(
                                                                                        [
                                                                                            (
                                                                                                'text',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    label='Text'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'link',
                                                                                                wagtail.blocks.StructBlock(
                                                                                                    [
                                                                                                        (
                                                                                                            'target',
                                                                                                            wagtail.blocks.StreamBlock(
                                                                                                                [
                                                                                                                    (
                                                                                                                        'page',
                                                                                                                        wagtail.blocks.PageChooserBlock(
                                                                                                                            icon='doc-empty-inverse',
                                                                                                                            label='Page',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'document',
                                                                                                                        wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                                            icon='doc-full',
                                                                                                                            label='Document',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'image',
                                                                                                                        wagtail.images.blocks.ImageChooserBlock(
                                                                                                                            label='Image'
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'url',
                                                                                                                        wagtail.blocks.URLBlock(
                                                                                                                            label='External link'
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'anchor',
                                                                                                                        wagtail.blocks.CharBlock(
                                                                                                                            help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                                            label='Anchor link',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                ],
                                                                                                                required=True,
                                                                                                            ),
                                                                                                        )
                                                                                                    ],
                                                                                                    required=True,
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'style',
                                                                                                wagtail.blocks.ChoiceBlock(
                                                                                                    choices=[
                                                                                                        (
                                                                                                            'primary',
                                                                                                            'Primary',
                                                                                                        ),
                                                                                                        (
                                                                                                            'secondary',
                                                                                                            'Secondary',
                                                                                                        ),
                                                                                                        (
                                                                                                            'primary-lg',
                                                                                                            'Large primary',
                                                                                                        ),
                                                                                                    ]
                                                                                                ),
                                                                                            ),
                                                                                        ],
                                                                                        default_style='primary-lg',
                                                                                    ),
                                                                                ),
                                                                                (
                                                                                    'image_block',
                                                                                    wagtail.blocks.StructBlock(
                                                                                        [
                                                                                            (
                                                                                                'image',
                                                                                                wagtail.images.blocks.ImageChooserBlock(
                                                                                                    label='Image'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'caption',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    label='Caption',
                                                                                                    required=False,
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'link',
                                                                                                wagtail.blocks.StructBlock(
                                                                                                    [
                                                                                                        (
                                                                                                            'target',
                                                                                                            wagtail.blocks.StreamBlock(
                                                                                                                [
                                                                                                                    (
                                                                                                                        'page',
                                                                                                                        wagtail.blocks.PageChooserBlock(
                                                                                                                            icon='doc-empty-inverse',
                                                                                                                            label='Page',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'document',
                                                                                                                        wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                                            icon='doc-full',
                                                                                                                            label='Document',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'image',
                                                                                                                        wagtail.images.blocks.ImageChooserBlock(
                                                                                                                            label='Image'
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'url',
                                                                                                                        wagtail.blocks.URLBlock(
                                                                                                                            label='External link'
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                    (
                                                                                                                        'anchor',
                                                                                                                        wagtail.blocks.CharBlock(
                                                                                                                            help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                                            label='Anchor link',
                                                                                                                        ),
                                                                                                                    ),
                                                                                                                ],
                                                                                                                required=False,
                                                                                                            ),
                                                                                                        )
                                                                                                    ],
                                                                                                    required=False,
                                                                                                ),
                                                                                            ),
                                                                                        ]
                                                                                    ),
                                                                                ),
                                                                            ]
                                                                        ),
                                                                        collapsed=True,
                                                                        form_classname='columns-block-list',
                                                                        label='Columns',
                                                                    ),
                                                                ),
                                                                (
                                                                    'horizontal_align',
                                                                    wagtail.blocks.ChoiceBlock(
                                                                        choices=[
                                                                            (
                                                                                'start',
                                                                                'Left',
                                                                            ),
                                                                            (
                                                                                'center',
                                                                                'Center',
                                                                            ),
                                                                            (
                                                                                'end',
                                                                                'Right',
                                                                            ),
                                                                        ],
                                                                        label='Horizontal alignment',
                                                                        required=False,
                                                                    ),
                                                                ),
                                                            ]
                                                        ),
                                                    ),
                                                ],
                                                label='Content',
                                            ),
                                        ),
                                        (
                                            'style',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    ('centered', 'Centered'),
                                                    (
                                                        'responsive',
                                                        'Responsive',
                                                    ),
                                                    ('dark', 'Dark'),
                                                ]
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                'columns_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'columns',
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.StreamBlock(
                                                    [
                                                        (
                                                            'title_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'text',
                                                                        wagtail.blocks.CharBlock(
                                                                            form_classname='title',
                                                                            label='Title',
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'level',
                                                                        wagtail.blocks.ChoiceBlock(
                                                                            choices=[
                                                                                (
                                                                                    2,
                                                                                    'Level 2 heading',
                                                                                ),
                                                                                (
                                                                                    3,
                                                                                    'Level 3 heading',
                                                                                ),
                                                                                (
                                                                                    4,
                                                                                    'Level 4 heading',
                                                                                ),
                                                                                (
                                                                                    5,
                                                                                    'Level 5 heading',
                                                                                ),
                                                                            ],
                                                                            label='Level',
                                                                        ),
                                                                    ),
                                                                ]
                                                            ),
                                                        ),
                                                        (
                                                            'paragraph_block',
                                                            wagtail_cblocks.blocks.ParagraphBlock(),
                                                        ),
                                                        (
                                                            'button_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'text',
                                                                        wagtail.blocks.CharBlock(
                                                                            label='Text'
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'link',
                                                                        wagtail.blocks.StructBlock(
                                                                            [
                                                                                (
                                                                                    'target',
                                                                                    wagtail.blocks.StreamBlock(
                                                                                        [
                                                                                            (
                                                                                                'page',
                                                                                                wagtail.blocks.PageChooserBlock(
                                                                                                    icon='doc-empty-inverse',
                                                                                                    label='Page',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'document',
                                                                                                wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                    icon='doc-full',
                                                                                                    label='Document',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'image',
                                                                                                wagtail.images.blocks.ImageChooserBlock(
                                                                                                    label='Image'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'url',
                                                                                                wagtail.blocks.URLBlock(
                                                                                                    label='External link'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'anchor',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                    label='Anchor link',
                                                                                                ),
                                                                                            ),
                                                                                        ],
                                                                                        required=True,
                                                                                    ),
                                                                                )
                                                                            ],
                                                                            required=True,
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'style',
                                                                        wagtail.blocks.ChoiceBlock(
                                                                            choices=[
                                                                                (
                                                                                    'primary',
                                                                                    'Primary',
                                                                                ),
                                                                                (
                                                                                    'secondary',
                                                                                    'Secondary',
                                                                                ),
                                                                                (
                                                                                    'primary-lg',
                                                                                    'Large primary',
                                                                                ),
                                                                            ]
                                                                        ),
                                                                    ),
                                                                ],
                                                                default_style='primary-lg',
                                                            ),
                                                        ),
                                                        (
                                                            'image_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'image',
                                                                        wagtail.images.blocks.ImageChooserBlock(
                                                                            label='Image'
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'caption',
                                                                        wagtail.blocks.CharBlock(
                                                                            label='Caption',
                                                                            required=False,
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'link',
                                                                        wagtail.blocks.StructBlock(
                                                                            [
                                                                                (
                                                                                    'target',
                                                                                    wagtail.blocks.StreamBlock(
                                                                                        [
                                                                                            (
                                                                                                'page',
                                                                                                wagtail.blocks.PageChooserBlock(
                                                                                                    icon='doc-empty-inverse',
                                                                                                    label='Page',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'document',
                                                                                                wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                    icon='doc-full',
                                                                                                    label='Document',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'image',
                                                                                                wagtail.images.blocks.ImageChooserBlock(
                                                                                                    label='Image'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'url',
                                                                                                wagtail.blocks.URLBlock(
                                                                                                    label='External link'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'anchor',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                    label='Anchor link',
                                                                                                ),
                                                                                            ),
                                                                                        ],
                                                                                        required=False,
                                                                                    ),
                                                                                )
                                                                            ],
                                                                            required=False,
                                                                        ),
                                                                    ),
                                                                ]
                                                            ),
                                                        ),
                                                    ]
                                                ),
                                                collapsed=True,
                                                form_classname='columns-block-list',
                                                label='Columns',
                                            ),
                                        ),
                                        (
                                            'horizontal_align',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    ('start', 'Left'),
                                                    ('center', 'Center'),
                                                    ('end', 'Right'),
                                                ],
                                                label='Horizontal alignment',
                                                required=False,
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                'row_columns_block',
                                wagtail.blocks.StructBlock(
                                    [
                                        (
                                            'columns',
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.StreamBlock(
                                                    [
                                                        (
                                                            'title_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'text',
                                                                        wagtail.blocks.CharBlock(
                                                                            form_classname='title',
                                                                            label='Title',
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'level',
                                                                        wagtail.blocks.ChoiceBlock(
                                                                            choices=[
                                                                                (
                                                                                    2,
                                                                                    'Level 2 heading',
                                                                                ),
                                                                                (
                                                                                    3,
                                                                                    'Level 3 heading',
                                                                                ),
                                                                                (
                                                                                    4,
                                                                                    'Level 4 heading',
                                                                                ),
                                                                                (
                                                                                    5,
                                                                                    'Level 5 heading',
                                                                                ),
                                                                            ],
                                                                            label='Level',
                                                                        ),
                                                                    ),
                                                                ]
                                                            ),
                                                        ),
                                                        (
                                                            'paragraph_block',
                                                            wagtail_cblocks.blocks.ParagraphBlock(),
                                                        ),
                                                        (
                                                            'button_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'text',
                                                                        wagtail.blocks.CharBlock(
                                                                            label='Text'
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'link',
                                                                        wagtail.blocks.StructBlock(
                                                                            [
                                                                                (
                                                                                    'target',
                                                                                    wagtail.blocks.StreamBlock(
                                                                                        [
                                                                                            (
                                                                                                'page',
                                                                                                wagtail.blocks.PageChooserBlock(
                                                                                                    icon='doc-empty-inverse',
                                                                                                    label='Page',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'document',
                                                                                                wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                    icon='doc-full',
                                                                                                    label='Document',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'image',
                                                                                                wagtail.images.blocks.ImageChooserBlock(
                                                                                                    label='Image'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'url',
                                                                                                wagtail.blocks.URLBlock(
                                                                                                    label='External link'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'anchor',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                    label='Anchor link',
                                                                                                ),
                                                                                            ),
                                                                                        ],
                                                                                        required=True,
                                                                                    ),
                                                                                )
                                                                            ],
                                                                            required=True,
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'style',
                                                                        wagtail.blocks.ChoiceBlock(
                                                                            choices=[
                                                                                (
                                                                                    'primary',
                                                                                    'Primary',
                                                                                ),
                                                                                (
                                                                                    'secondary',
                                                                                    'Secondary',
                                                                                ),
                                                                                (
                                                                                    'primary-lg',
                                                                                    'Large primary',
                                                                                ),
                                                                            ]
                                                                        ),
                                                                    ),
                                                                ],
                                                                default_style='primary-lg',
                                                            ),
                                                        ),
                                                        (
                                                            'image_block',
                                                            wagtail.blocks.StructBlock(
                                                                [
                                                                    (
                                                                        'image',
                                                                        wagtail.images.blocks.ImageChooserBlock(
                                                                            label='Image'
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'caption',
                                                                        wagtail.blocks.CharBlock(
                                                                            label='Caption',
                                                                            required=False,
                                                                        ),
                                                                    ),
                                                                    (
                                                                        'link',
                                                                        wagtail.blocks.StructBlock(
                                                                            [
                                                                                (
                                                                                    'target',
                                                                                    wagtail.blocks.StreamBlock(
                                                                                        [
                                                                                            (
                                                                                                'page',
                                                                                                wagtail.blocks.PageChooserBlock(
                                                                                                    icon='doc-empty-inverse',
                                                                                                    label='Page',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'document',
                                                                                                wagtail.documents.blocks.DocumentChooserBlock(
                                                                                                    icon='doc-full',
                                                                                                    label='Document',
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'image',
                                                                                                wagtail.images.blocks.ImageChooserBlock(
                                                                                                    label='Image'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'url',
                                                                                                wagtail.blocks.URLBlock(
                                                                                                    label='External link'
                                                                                                ),
                                                                                            ),
                                                                                            (
                                                                                                'anchor',
                                                                                                wagtail.blocks.CharBlock(
                                                                                                    help_text='An anchor in the current page, for example: <code>#target-id</code>.',
                                                                                                    label='Anchor link',
                                                                                                ),
                                                                                            ),
                                                                                        ],
                                                                                        required=False,
                                                                                    ),
                                                                                )
                                                                            ],
                                                                            required=False,
                                                                        ),
                                                                    ),
                                                                ]
                                                            ),
                                                        ),
                                                    ]
                                                ),
                                                collapsed=True,
                                                form_classname='columns-block-list',
                                                label='Columns',
                                            ),
                                        ),
                                        (
                                            'horizontal_align',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    ('start', 'Left'),
                                                    ('center', 'Center'),
                                                    ('end', 'Right'),
                                                ],
                                                label='Horizontal alignment',
                                                required=False,
                                            ),
                                        ),
                                        (
                                            'layout',
                                            wagtail.blocks.ChoiceBlock(
                                                choices=[
                                                    ('2', '2-columns'),
                                                    ('3', '3-columns'),
                                                    ('4', '4-columns'),
                                                    ('auto', 'natural width'),
                                                ]
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                        ],
                        use_json_field=True,
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



from django.conf import settings
from django.urls import include, path

from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.documents import urls as wagtaildocs_urls

urlpatterns = [
    path('admin/', include(wagtailadmin_urls)),
    path('documents/', include(wagtaildocs_urls)),
    path('', include(wagtail_urls)),
]

if settings.DEBUG:
    from django.conf.urls.static import static

    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from wagtail.admin.panels import FieldPanel
from wagtail.blocks import ChoiceBlock, StreamBlock
from wagtail.fields import StreamField
from wagtail.models import Page

from wagtail_cblocks.blocks import (
    ButtonBlock,
    ColumnsBlock,
    HeadingBlock,
    ImageBlock,
    ParagraphBlock,
    Style,
    StylizedStructBlock,
)

BUTTON_STYLES = [
    Style('primary', "Primary", 'btn-primary'),
    Style('secondary', "Secondary", 'btn-secondary'),
    Style('primary-lg', "Large primary", 'btn-primary btn-lg'),
]
BUTTON_DEFAULT_STYLE = 'primary-lg'


class BaseBlock(StreamBlock):
    title_block = HeadingBlock()
    paragraph_block = ParagraphBlock()
    button_block = ButtonBlock(
        styles=BUTTON_STYLES, default_style=BUTTON_DEFAULT_STYLE
    )
    image_block = ImageBlock()


class RowColumnsBlock(ColumnsBlock):
    LAYOUT_CHOICES = [
        ('2', "2-columns"),
        ('3', "3-columns"),
        ('4', "4-columns"),
        ('auto', "natural width"),
    ]

    layout = ChoiceBlock(choices=LAYOUT_CHOICES, default='auto')

    class Meta:
        label = "Row columns"
        template = 'tests/blocks/row_columns_block.html'
        column_block = BaseBlock()


class HeroBlock(StylizedStructBlock):
    blocks = BaseBlock(
        local_blocks=[('columns_block', ColumnsBlock(BaseBlock()))],
        label="Content",
    )

    styles = [
        Style('centered', "Centered", 'my-5 text-center hero-centered'),
        Style('responsive', "Responsive", 'container col-xxl-8'),
        Style('dark', "Dark", 'bg-dark text-white text-center'),
    ]

    class Meta:
        label = "Hero"
        template = 'tests/blocks/hero_block.html'


class BodyBlock(BaseBlock):
    hero_block = HeroBlock()
    columns_block = ColumnsBlock(BaseBlock())
    row_columns_block = RowColumnsBlock()


class StandardPage(Page):
    body = StreamField(BodyBlock(), use_json_field=True)

    content_panels = Page.content_panels + [
        FieldPanel('body'),
    ]
